@@ -25,7 +25,7 @@
 
 static int			 yylex(void);
 static int			 yyparse(void);
-static int printflike(1,2)	 yyerror(const char *, ...);
+static void printflike(1,2)	 yyerror(const char *, ...);
 
 static char			*yylex_token(int);
 static char			*yylex_format(void);
@@ -330,7 +330,7 @@ short *yysslim;
 YYSTYPE *yyvs;
 unsigned int yystacksize;
 int yyparse(void);
-#line 575 "cmd-parse.y"
+#line 589 "cmd-parse.y"
 
 static char *
 cmd_parse_get_error(const char *file, u_int line, const char *error)
@@ -594,7 +594,7 @@ cmd_parse_build_command(struct cmd_parse_command *cmd,
 		count++;
 	}
 
-	add = cmd_parse(values, count, pi->file, pi->line, &cause);
+	add = cmd_parse(values, count, pi->file, pi->line, pi->flags, &cause);
 	if (add == NULL) {
 		pr->status = CMD_PARSE_ERROR;
 		pr->error = cmd_parse_get_error(pi->file, pi->line, cause);
@@ -872,7 +872,7 @@ cmd_parse_from_arguments(struct args_value *values, u_int count,
 	return (&pr);
 }
 
-static int printflike(1, 2)
+static void printflike(1, 2)
 yyerror(const char *fmt, ...)
 {
 	struct cmd_parse_state	*ps = &parse_state;
@@ -881,7 +881,7 @@ yyerror(const char *fmt, ...)
 	char			*error;
 
 	if (ps->error != NULL)
-		return (0);
+		return;
 
 	va_start(ap, fmt);
 	xvasprintf(&error, fmt, ap);
@@ -889,7 +889,6 @@ yyerror(const char *fmt, ...)
 
 	ps->error = cmd_parse_get_error(pi->file, pi->line, error);
 	free(error);
-	return (0);
 }
 
 static int
@@ -1372,6 +1371,7 @@ yylex_token_tilde(char **buf, size_t *len)
 static char *
 yylex_token(int ch)
 {
+	struct cmd_parse_state	*ps = &parse_state;
 	char			*buf;
 	size_t			 len;
 	enum { START,
@@ -1395,9 +1395,12 @@ yylex_token(int ch)
 				ch = '\r';
 			}
 		}
-		if (state == NONE && ch == '\n') {
-			log_debug("%s: end at EOL", __func__);
-			break;
+		if (ch == '\n') {
+			if (state == NONE) {
+				log_debug("%s: end at EOL", __func__);
+				break;
+			}
+			ps->input->line++;
 		}
 
 		/* Whitespace or ; or } ends a token unless inside quotes. */
@@ -1491,7 +1494,7 @@ error:
 	free(buf);
 	return (NULL);
 }
-#line 1487 "cmd-parse.c"
+#line 1490 "cmd-parse.c"
 /* allocate initial stack or double stack size, up to YYMAXDEPTH */
 static int yygrowstack(void)
 {
@@ -1787,27 +1790,41 @@ case 14:
 {
 			struct cmd_parse_state	*ps = &parse_state;
 			int			 flags = ps->input->flags;
+			int			 flag = 1;
+			struct cmd_parse_scope	*scope;
 
-			if ((~flags & CMD_PARSE_PARSEONLY) &&
-			    (ps->scope == NULL || ps->scope->flag))
+			if (ps->scope != NULL) {
+				flag = ps->scope->flag;
+				TAILQ_FOREACH(scope, &ps->stack, entry)
+					flag = flag && scope->flag;
+			}
+
+			if ((~flags & CMD_PARSE_PARSEONLY) && flag)
 				environ_put(global_environ, yyvsp[0].token, 0);
 			free(yyvsp[0].token);
 		}
 break;
 case 15:
-#line 234 "cmd-parse.y"
+#line 241 "cmd-parse.y"
 {
 			struct cmd_parse_state	*ps = &parse_state;
 			int			 flags = ps->input->flags;
+			int			 flag = 1;
+			struct cmd_parse_scope	*scope;
 
-			if ((~flags & CMD_PARSE_PARSEONLY) &&
-			    (ps->scope == NULL || ps->scope->flag))
+			if (ps->scope != NULL) {
+				flag = ps->scope->flag;
+				TAILQ_FOREACH(scope, &ps->stack, entry)
+					flag = flag && scope->flag;
+			}
+
+			if ((~flags & CMD_PARSE_PARSEONLY) && flag)
 				environ_put(global_environ, yyvsp[0].token, ENVIRON_HIDDEN);
 			free(yyvsp[0].token);
 		}
 break;
 case 16:
-#line 245 "cmd-parse.y"
+#line 259 "cmd-parse.y"
 {
 			struct cmd_parse_state	*ps = &parse_state;
 			struct cmd_parse_scope	*scope;
@@ -1822,7 +1839,7 @@ case 16:
 		}
 break;
 case 17:
-#line 259 "cmd-parse.y"
+#line 273 "cmd-parse.y"
 {
 			struct cmd_parse_state	*ps = &parse_state;
 			struct cmd_parse_scope	*scope;
@@ -1835,7 +1852,7 @@ case 17:
 		}
 break;
 case 18:
-#line 271 "cmd-parse.y"
+#line 285 "cmd-parse.y"
 {
 			struct cmd_parse_state	*ps = &parse_state;
 			struct cmd_parse_scope	*scope;
@@ -1849,7 +1866,7 @@ case 18:
 		}
 break;
 case 19:
-#line 284 "cmd-parse.y"
+#line 298 "cmd-parse.y"
 {
 			struct cmd_parse_state	*ps = &parse_state;
 
@@ -1860,7 +1877,7 @@ case 19:
 		}
 break;
 case 20:
-#line 294 "cmd-parse.y"
+#line 308 "cmd-parse.y"
 {
 			if (yyvsp[-3].flag)
 				yyval.commands = yyvsp[-1].commands;
@@ -1871,7 +1888,7 @@ case 20:
 		}
 break;
 case 21:
-#line 303 "cmd-parse.y"
+#line 317 "cmd-parse.y"
 {
 			if (yyvsp[-6].flag) {
 				yyval.commands = yyvsp[-4].commands;
@@ -1883,7 +1900,7 @@ case 21:
 		}
 break;
 case 22:
-#line 313 "cmd-parse.y"
+#line 327 "cmd-parse.y"
 {
 			if (yyvsp[-4].flag) {
 				yyval.commands = yyvsp[-2].commands;
@@ -1899,7 +1916,7 @@ case 22:
 		}
 break;
 case 23:
-#line 327 "cmd-parse.y"
+#line 341 "cmd-parse.y"
 {
 			if (yyvsp[-7].flag) {
 				yyval.commands = yyvsp[-5].commands;
@@ -1917,7 +1934,7 @@ case 23:
 		}
 break;
 case 24:
-#line 344 "cmd-parse.y"
+#line 358 "cmd-parse.y"
 {
 			if (yyvsp[-2].flag) {
 				yyval.elif.flag = 1;
@@ -1930,7 +1947,7 @@ case 24:
 		}
 break;
 case 25:
-#line 355 "cmd-parse.y"
+#line 369 "cmd-parse.y"
 {
 			if (yyvsp[-3].flag) {
 				yyval.elif.flag = 1;
@@ -1949,7 +1966,7 @@ case 25:
 		}
 break;
 case 26:
-#line 373 "cmd-parse.y"
+#line 387 "cmd-parse.y"
 {
 			struct cmd_parse_state	*ps = &parse_state;
 
@@ -1962,13 +1979,13 @@ case 26:
 		}
 break;
 case 27:
-#line 384 "cmd-parse.y"
+#line 398 "cmd-parse.y"
 {
 			yyval.commands = yyvsp[-1].commands;
 		}
 break;
 case 28:
-#line 388 "cmd-parse.y"
+#line 402 "cmd-parse.y"
 {
 			yyval.commands = yyvsp[-2].commands;
 			TAILQ_CONCAT(yyval.commands, yyvsp[0].commands, entry);
@@ -1976,7 +1993,7 @@ case 28:
 		}
 break;
 case 29:
-#line 394 "cmd-parse.y"
+#line 408 "cmd-parse.y"
 {
 			struct cmd_parse_state	*ps = &parse_state;
 
@@ -1992,13 +2009,13 @@ case 29:
 		}
 break;
 case 30:
-#line 408 "cmd-parse.y"
+#line 422 "cmd-parse.y"
 {
 			yyval.commands = yyvsp[0].commands;
 		}
 break;
 case 31:
-#line 413 "cmd-parse.y"
+#line 427 "cmd-parse.y"
 {
 			struct cmd_parse_state	*ps = &parse_state;
 
@@ -2008,7 +2025,7 @@ case 31:
 		}
 break;
 case 32:
-#line 421 "cmd-parse.y"
+#line 435 "cmd-parse.y"
 {
 			struct cmd_parse_state		*ps = &parse_state;
 			struct cmd_parse_argument	*arg;
@@ -2024,7 +2041,7 @@ case 32:
 		}
 break;
 case 33:
-#line 435 "cmd-parse.y"
+#line 449 "cmd-parse.y"
 {
 			struct cmd_parse_state		*ps = &parse_state;
 			struct cmd_parse_argument	*arg;
@@ -2043,7 +2060,7 @@ case 33:
 		}
 break;
 case 34:
-#line 453 "cmd-parse.y"
+#line 467 "cmd-parse.y"
 {
 			if (yyvsp[-2].flag)
 				yyval.commands = yyvsp[-1].commands;
@@ -2054,7 +2071,7 @@ case 34:
 		}
 break;
 case 35:
-#line 462 "cmd-parse.y"
+#line 476 "cmd-parse.y"
 {
 			if (yyvsp[-4].flag) {
 				yyval.commands = yyvsp[-3].commands;
@@ -2066,7 +2083,7 @@ case 35:
 		}
 break;
 case 36:
-#line 472 "cmd-parse.y"
+#line 486 "cmd-parse.y"
 {
 			if (yyvsp[-3].flag) {
 				yyval.commands = yyvsp[-2].commands;
@@ -2082,7 +2099,7 @@ case 36:
 		}
 break;
 case 37:
-#line 486 "cmd-parse.y"
+#line 500 "cmd-parse.y"
 {
 			if (yyvsp[-5].flag) {
 				yyval.commands = yyvsp[-4].commands;
@@ -2100,7 +2117,7 @@ case 37:
 		}
 break;
 case 38:
-#line 503 "cmd-parse.y"
+#line 517 "cmd-parse.y"
 {
 			if (yyvsp[-1].flag) {
 				yyval.elif.flag = 1;
@@ -2113,7 +2130,7 @@ case 38:
 		}
 break;
 case 39:
-#line 514 "cmd-parse.y"
+#line 528 "cmd-parse.y"
 {
 			if (yyvsp[-2].flag) {
 				yyval.elif.flag = 1;
@@ -2132,7 +2149,7 @@ case 39:
 		}
 break;
 case 40:
-#line 532 "cmd-parse.y"
+#line 546 "cmd-parse.y"
 {
 			yyval.arguments = xcalloc(1, sizeof *yyval.arguments);
 			TAILQ_INIT(yyval.arguments);
@@ -2141,14 +2158,14 @@ case 40:
 		}
 break;
 case 41:
-#line 539 "cmd-parse.y"
+#line 553 "cmd-parse.y"
 {
 			TAILQ_INSERT_HEAD(yyvsp[0].arguments, yyvsp[-1].argument, entry);
 			yyval.arguments = yyvsp[0].arguments;
 		}
 break;
 case 42:
-#line 545 "cmd-parse.y"
+#line 559 "cmd-parse.y"
 {
 			yyval.argument = xcalloc(1, sizeof *yyval.argument);
 			yyval.argument->type = CMD_PARSE_STRING;
@@ -2156,7 +2173,7 @@ case 42:
 		}
 break;
 case 43:
-#line 551 "cmd-parse.y"
+#line 565 "cmd-parse.y"
 {
 			yyval.argument = xcalloc(1, sizeof *yyval.argument);
 			yyval.argument->type = CMD_PARSE_STRING;
@@ -2164,7 +2181,7 @@ case 43:
 		}
 break;
 case 44:
-#line 557 "cmd-parse.y"
+#line 571 "cmd-parse.y"
 {
 			yyval.argument = xcalloc(1, sizeof *yyval.argument);
 			yyval.argument->type = CMD_PARSE_COMMANDS;
@@ -2172,20 +2189,20 @@ case 44:
 		}
 break;
 case 45:
-#line 564 "cmd-parse.y"
+#line 578 "cmd-parse.y"
 {
 				yyval.commands = yyvsp[-1].commands;
 			}
 break;
 case 46:
-#line 568 "cmd-parse.y"
+#line 582 "cmd-parse.y"
 {
 				yyval.commands = yyvsp[-2].commands;
 				TAILQ_CONCAT(yyval.commands, yyvsp[-1].commands, entry);
 				free(yyvsp[-1].commands);
 			}
 break;
-#line 2181 "cmd-parse.c"
+#line 2198 "cmd-parse.c"
     }
     yyssp -= yym;
     yystate = *yyssp;
